
;*********************************************
;** AT90S2313 Assembly					  	**
;** SED1520 display controller  demo	  	**
;** Set Host computer to 115200,N,8,1     	**
;** When Host recieves PageNr then Host   	**
;** May Transmit the byte values for the  	** 
;** 61 Bytes per page, 8 pages in total	  	**
;**                                       	**
;** By markus@vandijkenelektronica.nl 2003  **
;** With thanks to Stephen Yu for his 		**
;** AT89C51	  source code					**
;*********************************************


.include "2313def.inc" 


.def 	S 				= R0 		; Storage for the Status Register

.def 	temp			= R16
.def	temp2			= R17
.def	temp3			= R18

.def	temp4			= r19				
.def	rxbyte 			= r20		

.def	Command			= r21
.def	Data			= r22
.def	PageNr			= R23

.def 	RXByteCount		= R24
.def	ByteCount		= R25
.def	TxByte			= R26	




;############ Equs ###########################

.equ	clock =7372800 ; Atmel system clk
.equ	baudrate = 115200
.equ	baudkonstant = (clock/(16*baudrate))-1

; Port B is the Databus

; Port D is the output, this is the pinning
.equ	A0 		= 	2  ; PD2
.equ	RW	 	= 	4  ; PD4
.equ	EMaster = 	3  ; PD6
.equ	Eslave 	=   6  ; PD3
.equ	RES 	=   5  ; PD5 can be left out and attached to a

; Commands for the sed1520
.equ	OnOFF 			= 0b10101110 ; bit 0 indicates is on of off
.equ	SetStartLine 	= 0b11000000 ; bit 0 to 4 is the line Address
.equ	SetPageNr 		= 0b10111000 ; bit 0 to 1 is the page number
.equ	SetColumnAdr	= 0b00000000 ; bit 0 to 6 is the column Adress


.dseg
	PageBuffer:		.BYTE 61 ; display buffer , holds 1 page there are 8 pages




.cseg
;**** Interrupt Vectors ****

.org $0000
	rjmp	reset			; Reset handle
.org INT0addr
	reti
.org INT1addr
	reti
.org ICP1addr
	reti
;.org OC1Aaddr
	;reti
;.org OC1Baddr
	;reti
.org OVF1addr
	 reti	; rjmp 	TIM1_OVF				; timer 1 overflow int not used at the moment
.org OVF0addr
	 reti  	; rjmp 	TIM0_OVF                ; timer 0 overflow int not used at the moment
;.org SPIaddr
	;reti
.org URXCaddr
	rjmp   	uart_rxc						; service RX complete
.org UDREaddr
	reti
.org UTXCaddr
	reti
.org ACIaddr
	reti


;**** initialization ****

reset:
;############## setup stack pointer ###############
		ldi		temp, LOW(RAMEND)	; SP = RAMEND (=$025F)
		out		SPL, temp		; 

		;ldi	temp, HIGH(RAMEND)	; de-cooment if a 8515 is used
		;out	SPH, temp

;############## setup ports #######################		
		ser		temp
		out 	DDRB,temp              ; set port B as output


		ldi		temp,0b11111100 	; bit 5 = Enable, 6 = WR on port D is output
		out 	DDRD,temp			; bit 4 = Reg Select, bit3 = LED

;############### setup uart #########################
		ldi		temp,$98		; RXCIE=1, RXEN=1, TXEN=1
		out		UCR,temp		


		ldi		temp,baudkonstant	
		out		ubrr,temp	;load baudrate

Main:		


;###### Puls the reset Line, leave out is RES not attached to Atmel ##########
	


		sbi		PortD,RES 
		nop
		nop
		nop


		cbi		PortD,RES
		nop
		nop
		nop

		sbi		PortD,RES
		nop
		nop
		nop

;#####################################################################

		ldi		PageNr,0 ; start on overflow , causes Pagenr to reset
		ldi	    RXByteCount,0

		rcall	init


		rcall	FillCheckers

		ldi		TxByte,48
		rcall	SendRS232Byte



;############## enable interrupts      ################
		sei				; set Global Interrupt Enable

	
Foreverloop:

		; nothing to see here, move along, move along

		rjmp 	Foreverloop
;########################################################
;########################################################
;########################################################
;########################################################
;########################################################


COM_W_S:                         ; ( PROCEDURE FOR E2 COMMAND WRITE )

        cbi		PortD,A0 	;CLR     A0
        cbi		PortD,RW  	;CLR     R/W
        nop
        sbi		PortD,ESlave	;SETB    E2
        nop
        out     PortB,Command        ; output command
        nop
        cbi		PortD,ESlave	;CLR     E2
        nop
		sbi		PortD,RW  		;SETB    R/W
        sbi		PortD,A0    ; SetBit A0
        nop
        ret

DATA_W_S:                         ; ( PROCEDURE FOR E2 DATA WRITE )
        sbi		PortD,A0 	;CLR     A0
        cbi		PortD,RW  	;CLR     R/W
        nop
        sbi		PortD,ESlave	;SETB    E2
        nop
        out     PortB,data        ; output command
        nop
        cbi		PortD,ESlave	;CLR     E2
        nop
		sbi		PortD,RW  		;SETB    R/W
        cbi		PortD,A0    ; SetBit A0
        nop
        ret

COM_W_M:                         ; ( PROCEDURE FOR E2 COMMAND WRITE )

        cbi		PortD,A0 	;CLR     A0
        cbi		PortD,RW  	;CLR     R/W
        nop
        sbi		PortD,EMaster	;SETB    E2
        nop
        out     PortB,Command        ; output command
        nop
        cbi		PortD,EMaster	;CLR     E2
        nop
		sbi		PortD,RW  		;SETB    R/W
        sbi		PortD,A0    ; SetBit A0
        nop
        ret

DATA_W_M:                         ; ( PROCEDURE FOR E2 DATA WRITE )
        sbi		PortD,A0 	;CLR     A0
        cbi		PortD,RW  	;CLR     R/W
        nop
        sbi		PortD,EMaster	;SETB    E2
        nop
        out     PortB,data        ; output command
        nop
        cbi		PortD,EMaster	;CLR     E2
        nop
		sbi		PortD,RW  		;SETB    R/W
        cbi		PortD,A0    ; SetBit A0
        nop
        ret


INIT:   
		;ldi     Command,0b11100010	; RESET COMMAND, START LINE REGISTER ON 1ST LINE
        ;rcall	Com_W_M         	; COLUMN AND PAGE ADDRESS COUNTER = 0
        ;rcall	Com_W_S  
        ldi     Command,0b10100100 	; STATIC DRIVE OFF -> NORMAL DRIVING
        rcall	Com_W_M  
        rcall	Com_W_S  
        ldi     Command,0b10101001  ; SELECT DUTY : 1/32 DUTY
        rcall	Com_W_M  
        rcall	Com_W_S  
        ldi     Command,0b10100000 	; ADC : CW (FORWARD)
        rcall	Com_W_M  
        rcall	Com_W_S  
        ldi     Command,0b11000000   ; DISPLAY START LINE = COM 0
        rcall	Com_W_M  
        rcall	Com_W_S  
        ldi     Command,0b10101111    ; TURN DISPLAY ON
        rcall	Com_W_M  
        rcall	Com_W_S  
        ret

FillCheckers:
		ldi		YL,low(PageBuffer)  ; let Y point to start Step Sequence Buffer
		ldi		YH,high(PageBuffer)
		
		ldi		temp,30
Next_Beet:	
		ldi		temp2,0b11001100
		st		Y+,temp2
		ldi		temp2,0b00110011
		st		Y+,temp2
		dec		temp
		brne	Next_Beet

		ldi		temp2,0b11001100
		st		Y+,temp2
		ldi		temp2,0b00110011
		st		Y+,temp2


		
		ldi		PageNr,0b00000000
		rcall	WritePage

		ldi		PageNr,0b00000001
		rcall	WritePage

		ldi		PageNr,0b00000010
		rcall	WritePage

		ldi		PageNr,0b00000011
		rcall	WritePage
		ldi		PageNr,0b00000100
		rcall	WritePage

		ldi		PageNr,0b00000101
		rcall	WritePage

		ldi		PageNr,0b00000110
		rcall	WritePage

		ldi		PageNr,0b00000111
		rcall	WritePage

		ret




;###################################################################################################
;###################################################################################################
;###################################################################################################

WritePage: ; PageNr bit 0 and 1 are the page and bit 2 indicate master or slave (left or right)

		sbrc	PageNr,2  ; test if master or slave Skip
		rjmp	Do_Slave  ; if its set do slave
Do_Master:

		mov		Command,PageNr
		andi	Command,0b00000011 	; we only need the page Nr now, wipe all but first 2 bits
		ori		Command,SetPageNr 	; set the bits for SET Page

		rcall	Com_W_M

		clr		Command  				; command = 0
		ori		Command,SetColumnAdr    ; set column Address to 0
		rcall	Com_W_M

		ldi		YL,low(PageBuffer)  ; let Y point to start Step Sequence Buffer
		ldi		YH,high(PageBuffer)

		ldi		temp,61

Do_61_B_M:

		ld		data,Y+
		rcall	Data_W_M

		dec		temp
		brne	Do_61_B_M

		rjmp	End_WriteP


Do_slave:
		mov		Command,PageNr
		andi	Command,0b00000011 ; we only need the page Nr now
		ori		Command,SetPageNr ; set the bits for SET Page

		rcall	Com_W_S

		clr		Command  			; command = 0
		ori		Command,SetColumnAdr   ; 
		rcall	Com_W_S

Output_Bytes:
		ldi		YL,low(PageBuffer)  ; let Y point to start Step Sequence Buffer
		ldi		YH,high(PageBuffer)

		ldi		temp,61
Do_61_B_S:

		ld		data,Y+
		rcall	Data_W_S

		dec		temp
		brne	Do_61_B_S:

End_WriteP:


		ret
;***************************************************************************
;*
;* "uart_rxc"
;*
;* Description: Receives byte and puts it into screen buffer
;* uses r17 and r18,r19,r30,r31 (putintobuf)
;***************************************************************************

uart_rxc:
		sbic	USR, FE			; skip if framing error
		rjmp	frame_err		; 
		in		rxbyte, UDR		; get rx byte
		sbic	USR, OR			; skip if overrun error
		rjmp	overrun	; 	

Data_ok:


		rcall 	ProcessData

carry_on:	
		reti

frame_err:
		in		rxbyte, UDR		; read byte to clear UDR		
		reti					; return

overrun:
		in		rxbyte, UDR		; read byte to clear UDR
		reti					; return



;***********************
; Very simple byte send
; sends byte in temp
;***********************
SendRS232Byte:
TX_wait:
		sbi			UCR,TXEN
		sbis		USR,UDRE
		rjmp		TX_wait
		out			UDR,TxByte
		cbi			UCR,TXEN
		ret			;go back

Send_OK:
		ldi			TxByte,79
		rcall		SendRS232Byte
	
		ldi			TxByte,75
		rcall		SendRS232Byte

		ret

;#################################################
;# Fills Display buffer with 16 bytes then sends #
;# Host a char to indicate buffer pointer reset  #
;# So when host recieves the control char it     #
;# Knows the buffer pointer is pointing to the   #
;# first char in the display buffer              #
;#################################################

ProcessData:
		ldi			ZL,low(PageBuffer)  ; let Y point to start Step Sequence Buffer
		ldi			ZH,high(PageBuffer)	


		add			ZL,RXByteCount
		st			Z,RXByte

		inc			RXByteCount
		cpi			RXbyteCount,61
		brlo		Just_Stored_it

		ldi			RXByteCount,0  ; resetcounter
		
Send_RST_Flag:

		inc			PageNr
		cpi			PageNr,0b00001000
		brlo		No_overflow
		clr			PageNr ; resets it 0
No_overflow:
		
		rcall		WritePage
		
		ldi			TxByte,48
		andi		TXByte,0b11111000
		or			TXbyte,PageNr
		rcall		SendRS232Byte
		

Just_Stored_it:
		
		ret



